@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: DataPool.cpp,v 1.14 2007/03/25 20:48:29 leonb Exp $
-// $Name: release_3_5_20_2 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -539,34 +536,36 @@ DataPool::BlockList::add_range(int start, int length)
 	 int size=list[pos];
 	 block_end=block_start+abs(size);
 	 if (size<0)
-	    if (block_start<start)
-	    {
-	       if (block_end>start && block_end<=start+length)
+	   {
+	     if (block_start<start)
 	       {
-		  list[pos]=-(start-block_start);
-		  list.insert_after(pos, block_end-start);
-		  ++pos;
-		  block_start=start;
-	       } else if (block_end>start+length)
+		 if (block_end>start && block_end<=start+length)
+		   {
+		     list[pos]=-(start-block_start);
+		     list.insert_after(pos, block_end-start);
+		     ++pos;
+		     block_start=start;
+		   } else if (block_end>start+length)
+		   {
+		     list[pos]=-(start-block_start);
+		     list.insert_after(pos, length);
+		     ++pos;
+		     list.insert_after(pos, -(block_end-(start+length)));
+		     ++pos;
+		     block_start=start+length;
+		   }
+	       } else if (block_start>=start && block_start<start+length)
 	       {
-		  list[pos]=-(start-block_start);
-		  list.insert_after(pos, length);
-		  ++pos;
-		  list.insert_after(pos, -(block_end-(start+length)));
-		  ++pos;
-		  block_start=start+length;
+		 if (block_end<=start+length) list[pos]=abs(size);
+		 else
+		   {
+		     list[pos]=start+length-block_start;
+		     list.insert_after(pos, -(block_end-(start+length)));
+		     ++pos;
+		     block_start=start+length;
+		   }
 	       }
-	    } else if (block_start>=start && block_start<start+length)
-	    {
-	       if (block_end<=start+length) list[pos]=abs(size);
-	       else
-	       {
-		  list[pos]=start+length-block_start;
-		  list.insert_after(pos, -(block_end-(start+length)));
-		  ++pos;
-		  block_start=start+length;
-	       }
-	    }
+	   }
 	 block_start=block_end;
 	 ++pos;
       }
@@ -582,15 +581,15 @@ DataPool::BlockList::add_range(int start, int length)
       {
 	 GPosition pos1=pos; ++pos1;
 	 while(pos1)
-	 {
-	    if (list[pos]<0 && list[pos1]>0 ||
-		list[pos]>0 && list[pos1]<0)
+	   {
+	     if ( (list[pos]<0 && list[pos1]>0) ||
+		  (list[pos]>0 && list[pos1]<0) )
 	       break;
-	    list[pos]+=list[pos1];
-	    GPosition this_pos=pos1;
-	    ++pos1;
-	    list.del(this_pos);
-	 }
+	     list[pos]+=list[pos1];
+	     GPosition this_pos=pos1;
+	     ++pos1;
+	     list.del(this_pos);
+	   }
 	 pos=pos1;
       }
    } // if (length>0)
@@ -615,18 +614,21 @@ DataPool::BlockList::get_bytes(int start, int length) const
       int size=list[pos];
       block_end=block_start+abs(size);
       if (size>0)
-	 if (block_start<start)
-	 {
-	    if (block_end>=start && block_end<start+length)
-	       bytes+=block_end-start;
-            else if (block_end>=start+length)
-	       bytes+=length;
-	 } else
-	 {
-	    if (block_end<=start+length)
-	       bytes+=block_end-block_start;
-	    else bytes+=start+length-block_start;
-	 }
+	{
+	  if (block_start<start)
+	    {
+	      if (block_end>=start && block_end<start+length)
+		bytes+=block_end-start;
+	      else if (block_end>=start+length)
+		bytes+=length;
+	    }
+	  else
+	    {
+	      if (block_end<=start+length)
+		bytes+=block_end-block_start;
+	      else bytes+=start+length-block_start;
+	    }
+	}
       block_start=block_end;
    }
    return bytes;
@@ -648,16 +650,20 @@ DataPool::BlockList::get_range(int start, int length) const
    GCriticalSectionLock lk((GCriticalSection *) &lock);
    int block_start=0, block_end=0;
    for(GPosition pos=list;pos && block_start<start+length;++pos)
-   {
-      int size=list[pos];
-      block_end=block_start+abs(size);
-      if (block_start<=start && block_end>start)
-	 if (size<0) return -1;
-         else
-	    if (block_end>start+length) return length;
-            else return block_end-start;
-      block_start=block_end;
-   }
+     {
+       int size=list[pos];
+       block_end=block_start+abs(size);
+       if (block_start<=start && block_end>start)
+	 {
+	   if (size<0)
+	     return -1;
+	   else if (block_end>start+length)
+	     return length;
+	   else
+	     return block_end-start;
+	 }
+       block_start=block_end;
+     }
    return 0;
 }
 
@@ -835,10 +841,10 @@ DataPool::~DataPool(void)
 
   clear_stream(true);
   if (furl.is_local_file_url())
-  {
-    FCPools::get()->del_pool(furl, this);
-  }
-   
+    if (this->get_count() > 1)
+      FCPools::get()->del_pool(furl, this);
+  
+  GP<DataPool> pool = this->pool;
   {
 	 // Wait until the static_trigger_cb() exits
       GCriticalSectionLock lock(&trigger_lock);
@@ -846,7 +852,6 @@ DataPool::~DataPool(void)
         pool->del_trigger(static_trigger_cb, this);
       del_trigger(static_trigger_cb, this);
   }
-
   if (pool)
   {
       GCriticalSectionLock lock(&triggers_lock);
@@ -984,7 +989,9 @@ DataPool::get_size(int dstart, int dlength) const
       if (dlength<0) return 0;
    }
    
-   if (pool) return pool->get_size(start+dstart, dlength);
+   GP<DataPool> pool = this->pool;
+   if (pool) 
+     return pool->get_size(start+dstart, dlength);
    else if (furl.is_local_file_url())
    {
       if (start+dstart+dlength>length) return length-(start+dstart);
@@ -1120,48 +1127,50 @@ DataPool::get_data(void * buffer, int offset, int sz, int level)
    
    if (! sz)
      return 0;
-   
+
+   GP<DataPool> pool = this->pool;
    if (pool)
      {
        DEBUG_MSG("DataPool::get_data(): from pool\n");
        DEBUG_MAKE_INDENT(3);
        int retval=0;
        if (length>0 && offset+sz>length)
-         sz=length-offset;
+	 sz=length-offset;
        if (sz<0)
-        sz=0;
+	 sz=0;
        for(;;)
-         {
-           // Ask the underlying (master) DataPool for data. Note, that
-           // master DataPool may throw the "DATA_POOL_REENTER" exception
-           // demanding all readers to restart. This happens when
-           // a DataPool in the chain of DataPools stops. All readers
-           // should return to the most upper level and then reenter the
-           // DataPools hierarchy. Some of them will be stopped by
-           // DataPool::Stop exception.
-           G_TRY
-             {
-               if(stop_flag||stop_blocked_flag&&!is_eof()&&!has_data(offset, sz))
-                 G_THROW( DataPool::Stop );
-               retval=pool->get_data(buffer, start+offset, sz, level+1);
-             } 
-           G_CATCH(exc) 
-             {
-               pool->clear_stream(true);
-               if ((exc.get_cause() != GUTF8String( ERR_MSG("DataPool.reenter") ) ) 
-                   || level)
-                 G_RETHROW;
-             } G_ENDCATCH;
-           pool->clear_stream(true);
-           return retval;
-         }
+	 {
+	   // Ask the underlying (master) DataPool for data. Note, that
+	   // master DataPool may throw the "DATA_POOL_REENTER" exception
+	   // demanding all readers to restart. This happens when
+	   // a DataPool in the chain of DataPools stops. All readers
+	   // should return to the most upper level and then reenter the
+	   // DataPools hierarchy. Some of them will be stopped by
+	   // DataPool::Stop exception.
+	   G_TRY
+	     {
+	       if(stop_flag
+		  || (stop_blocked_flag && !is_eof() && !has_data(offset, sz)))
+		 G_THROW( DataPool::Stop );
+	       retval=pool->get_data(buffer, start+offset, sz, level+1);
+	     }
+	   G_CATCH(exc)
+	   {
+	     pool->clear_stream(true);
+	     if ((exc.get_cause() != GUTF8String(ERR_MSG("DataPool.reenter")))
+		 || level)
+	       G_RETHROW;
+	   } G_ENDCATCH;
+	   pool->clear_stream(true);
+	   return retval;
+	 }
      }
    else if (furl.is_local_file_url())
      {
        DEBUG_MSG("DataPool::get_data(): from file\n");
        DEBUG_MAKE_INDENT(3);
        if (length>0 && offset+sz>length)
-         sz=length-offset;
+	 sz=length-offset;
        if (sz<0)
          sz=0;
        
@@ -1340,6 +1349,7 @@ DataPool::stop(bool only_blocked)
       // thru this DataPool again and will be stopped (DataPool::Stop exception)
       // Others (which entered the master DataPool thru other slave DataPools)
       // will simply continue waiting for their data.
+   GP<DataPool> pool = this->pool;
    if (pool)
    {
 	 // This loop is necessary because there may be another thread, which
@@ -1431,52 +1441,53 @@ DataPool::check_triggers(void)
 {
   DEBUG_MSG("DataPool::check_triggers(): calling activated trigger callbacks.\n");
   DEBUG_MAKE_INDENT(3);
-  
+
   if (!pool && !furl.is_local_file_url())
     while(true)
-    {
-      GP<Trigger> trigger;
-      
-      // First find a candidate (trigger, which needs to be called)
-      // Don't remove it from the list yet. del_trigger() should
-      // be able to find it if necessary and disable.
       {
-        GCriticalSectionLock list_lock(&triggers_lock);
-        for(GPosition pos=triggers_list;pos;++pos)
-        {
-          GP<Trigger> t=triggers_list[pos];
-          if (is_eof() || t->length>=0 &&
-            block_list->get_bytes(t->start, t->length)==t->length)
-          {
-            trigger=t;
-            break;
-          }
-        }
+	GP<Trigger> trigger;
+
+	// First find a candidate (trigger, which needs to be called)
+	// Don't remove it from the list yet. del_trigger() should
+	// be able to find it if necessary and disable.
+	{
+	  GCriticalSectionLock list_lock(&triggers_lock);
+	  for(GPosition pos=triggers_list;pos;++pos)
+	    {
+	      GP<Trigger> t=triggers_list[pos];
+	      if (is_eof()
+		  || (t->length>=0 &&
+		      block_list->get_bytes(t->start, t->length)==t->length))
+		{
+		  trigger=t;
+		  break;
+		}
+	    }
+	}
+
+	if (trigger)
+	  {
+	    // Now check that the trigger is not disabled
+	    // and lock the trigger->disabled lock for the duration
+	    // of the trigger. This will block the del_trigger() and
+	    // will postpone client's destruction (usually following
+	    // the call to del_trigger())
+	    {
+	      GMonitorLock lock(&trigger->disabled);
+	      if (!trigger->disabled)
+		call_callback(trigger->callback, trigger->cl_data);
+	    }
+
+	    // Finally - remove the trigger from the list.
+	    GCriticalSectionLock list_lock(&triggers_lock);
+	    for(GPosition pos=triggers_list;pos;++pos)
+	      if (triggers_list[pos]==trigger)
+		{
+		  triggers_list.del(pos);
+		  break;
+		}
+	  } else break;
       }
-      
-      if (trigger)
-      {
-	       // Now check that the trigger is not disabled
-	       // and lock the trigger->disabled lock for the duration
-	       // of the trigger. This will block the del_trigger() and
-	       // will postpone client's destruction (usually following
-	       // the call to del_trigger())
-        {
-          GMonitorLock lock(&trigger->disabled);
-          if (!trigger->disabled)
-            call_callback(trigger->callback, trigger->cl_data);
-        }
-        
-	       // Finally - remove the trigger from the list.
-        GCriticalSectionLock list_lock(&triggers_lock);
-        for(GPosition pos=triggers_list;pos;++pos)
-          if (triggers_list[pos]==trigger)
-          {
-            triggers_list.del(pos);
-            break;
-          }
-      } else break;
-    }
 }
 
 void
@@ -1503,6 +1514,7 @@ DataPool::add_trigger(int tstart, int tlength,
         call_callback(callback, cl_data);
       }else
       {
+         GP<DataPool> pool = this->pool;
 	 if (pool)
 	 {
 	       // We're connected to a DataPool
@@ -1567,6 +1579,7 @@ DataPool::del_trigger(void (* callback)(void *), void * cl_data)
         break;
    }
 
+   GP<DataPool> pool = this->pool;
    if (pool)
      pool->del_trigger(callback, cl_data);
 }
@@ -1592,6 +1605,7 @@ DataPool::trigger_cb(void)
    DEBUG_MSG("DataPool::trigger_cb() called\n");
    DEBUG_MAKE_INDENT(3);
 
+   GP<DataPool> pool = this->pool;
    if (pool)
    {
       // Connected to a pool
@@ -1768,6 +1782,7 @@ void
 DataPool::close_all(void)
 {
   OpenFiles::get()->close_all();
+  FCPools::get()->clean();
 }
 
 
