@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: DjVuDumpHelper.h,v 1.11 2007/05/19 03:07:33 leonb Exp $
-// $Name: release_3_5_20_2 $
 
 #ifndef _DJVUDUMPHELPER_H
 #define _DJVUDUMPHELPER_H
@@ -76,8 +73,7 @@
     @author
     L\'eon Bottou <leonb@research.att.com> -- as a separate program.\\
     Andrei Erofeev <eaf@geocities.com> -- as a class.
-    @version
-    #$Id: DjVuDumpHelper.h,v 1.11 2007/05/19 03:07:33 leonb Exp $# */
+*/
 //@{
 
 
