@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: DjVuFile.cpp,v 1.18 2009/08/18 22:33:44 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -1210,7 +1207,7 @@ DjVuFile::decode_chunk( const GUTF8String &id, const GP<ByteStream> &gbs,
       iffout.copy(achunk);
       iffout.close_chunk();
     }
-  else if (chkid == "CELX")
+  else if (chkid == "CELX" || chkid == "SINF")
     {
       G_THROW( ERR_MSG("DjVuFile.securedjvu") );
     }
