@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: DjVuGlobal.h,v 1.12 2007/05/19 03:07:33 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifndef _DJVUGLOBAL_H
 #define _DJVUGLOBAL_H
@@ -65,14 +62,12 @@
 # pragma interface
 #endif
 
-#if defined(UNDER_CE)
-# ifndef __WCEALT_H__
-inline void * operator new(size_t, void * ptr) { return ptr; }
-# endif
-#elif defined(AUTOCONF) && defined(HAVE_STDINCLUDES)
+#if defined(HAVE_STDINCLUDES)
 # include <new>
-#else
+#elif defined(HAVE_NEW_H)
 # include <new.h>
+#else
+# include <new> // try standard c++ anyway!
 #endif
 
 #ifdef WIN32
@@ -99,8 +94,6 @@ inline void * operator new(size_t, void * ptr) { return ptr; }
     
     @memo
     Global definitions.
-    @version
-    #$Id: DjVuGlobal.h,v 1.12 2007/05/19 03:07:33 leonb Exp $#
     @author
     L\'eon Bottou <leonb@research.att.com> -- empty file.\\
     Bill Riemers <docbill@sourceforge.net> -- real work.  */
