@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: DjVuImage.cpp,v 1.17 2008/08/06 04:40:41 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -668,7 +665,7 @@ DjVuImage::get_bitmap(const GRect &rect,
 
 GP<GPixmap>
 DjVuImage::get_bg_pixmap(const GRect &rect, 
-                         int subsample, double gamma) const
+                         int subsample, double gamma, GPixel white) const
 {
   GP<GPixmap> pm = 0;
   // Access image size
@@ -752,8 +749,9 @@ DjVuImage::get_bg_pixmap(const GRect &rect,
           ps.scale(xrect, *ipm, rect, *pm);
         }
       // Apply gamma correction
-      if (pm && gamma_correction!=1.0)
-        pm->color_correct(gamma_correction);
+      if (pm)
+        if (gamma_correction!=1.0 || white != GPixel::WHITE)
+          pm->color_correct(gamma_correction, white);
       return pm;
     }
 
@@ -796,8 +794,9 @@ DjVuImage::get_bg_pixmap(const GRect &rect,
           ps.scale(xrect, *bgpm, rect, *pm);
         }
       // Apply gamma correction
-      if (pm && gamma_correction!=1.0)
-        pm->color_correct(gamma_correction);
+      if (pm)
+        if (gamma_correction!=1.0 || white != GPixel::WHITE)
+          pm->color_correct(gamma_correction, white);
       return pm;
     }
 
@@ -806,10 +805,16 @@ DjVuImage::get_bg_pixmap(const GRect &rect,
 }
 
 
+GP<GPixmap>
+DjVuImage::get_bg_pixmap(const GRect &rect, int subsample, double gamma) const
+{
+  return get_bg_pixmap(rect, subsample, gamma, GPixel::WHITE);
+}
+
 
 int  
 DjVuImage::stencil(GPixmap *pm, const GRect &rect,
-		   int subsample, double gamma) const
+		   int subsample, double gamma, GPixel white) const
 {
   // Warping and blending. 
   if (!pm)
@@ -857,8 +862,8 @@ DjVuImage::stencil(GPixmap *pm, const GRect &rect,
           if (pshape.bits &&
               pblit->left <= rect.xmax * subsample &&
               pblit->bottom <= rect.ymax * subsample &&
-              pblit->left + (int)pshape.bits->columns() >= rect.xmin * subsample &&
-              pblit->bottom + (int)pshape.bits->rows() >= rect.ymin * subsample )
+              pblit->left+(int)pshape.bits->columns() >= rect.xmin*subsample &&
+              pblit->bottom+(int)pshape.bits->rows() >= rect.ymin*subsample )
             {
               // Record component list
               if (fgbc) components.append(blitno);
@@ -886,7 +891,7 @@ DjVuImage::stencil(GPixmap *pm, const GRect &rect,
       GTArray<GPixel> colors(0,palettesize-1);
       for (int i=0; i<palettesize; i++)
         fg->index_to_color(i, colors[i]);
-      GPixmap::color_correct(gamma_correction, colors, palettesize);
+      GPixmap::color_correct(gamma_correction, white, colors, palettesize);
       // Blit all components (one color at a time)
       while (components.size() > 0)
         {
@@ -938,7 +943,8 @@ DjVuImage::stencil(GPixmap *pm, const GRect &rect,
                        subsample);
             }
           // Blend color into background pixmap
-          pm->blit(bm, comprect.xmin-rect.xmin, comprect.ymin-rect.ymin, &colors[colorindex]);
+          pm->blit(bm, comprect.xmin-rect.xmin, comprect.ymin-rect.ymin, 
+                   &colors[colorindex]);
         }
       return 1;
     }
@@ -967,7 +973,7 @@ DjVuImage::stencil(GPixmap *pm, const GRect &rect,
       GRect provided(0,0,w,h);
       ps.scale(provided, *fgpm, rect, *nfg);
       // Attenuate background and blit
-      nfg->color_correct(gamma_correction);
+      nfg->color_correct(gamma_correction, white);
       pm->blend(bm, 0, 0, nfg); // blend == attenuate + blit
       return 1;
 #else 
@@ -988,7 +994,7 @@ DjVuImage::stencil(GPixmap *pm, const GRect &rect,
       if (red == wantedred)
         {
           // Simple foreground upsampling is enough.
-          pm->stencil(bm, fgpm, supersample, &rect, gamma_correction);
+          pm->stencil(bm, fgpm, supersample, &rect, gamma_correction, white);
           return 1;
         }
       else 
@@ -1020,7 +1026,7 @@ DjVuImage::stencil(GPixmap *pm, const GRect &rect,
               ps.scale(provided, *fgpm, desired, *nfg);
             }
           // Use combined warp+blend function
-          pm->stencil(bm, nfg, supersample, &rect, gamma_correction);
+          pm->stencil(bm, nfg, supersample, &rect, gamma_correction, white);
           // Cache
           tagimage = this;
           tagfgpm = fgpm;
@@ -1037,7 +1043,7 @@ DjVuImage::stencil(GPixmap *pm, const GRect &rect,
 
 GP<GPixmap>
 DjVuImage::get_fg_pixmap(const GRect &rect, 
-                         int subsample, double gamma) const
+                         int subsample, double gamma, GPixel white) const
 {
   // Obtain white background pixmap
   GP<GPixmap> pm;
@@ -1047,7 +1053,7 @@ DjVuImage::get_fg_pixmap(const GRect &rect,
   if (width && height)
   {
     pm = GPixmap::create(rect.height(),rect.width(), &GPixel::WHITE);
-    if (!stencil(pm, rect, subsample, gamma))
+    if (!stencil(pm, rect, subsample, gamma, white))
       pm=0;
   }
   return pm;
@@ -1055,12 +1061,21 @@ DjVuImage::get_fg_pixmap(const GRect &rect,
 
 
 GP<GPixmap>
-DjVuImage::get_pixmap(const GRect &rect, int subsample, double gamma) const
+DjVuImage::get_fg_pixmap(const GRect &rect, 
+                         int subsample, double gamma) const
+{
+  return get_fg_pixmap(rect, subsample, gamma, GPixel::WHITE);
+}
+
+
+GP<GPixmap>
+DjVuImage::get_pixmap(const GRect &rect, int subsample, 
+                      double gamma, GPixel white) const
 {
   // Get background
-  GP<GPixmap> pm = get_bg_pixmap(rect, subsample, gamma);
+  GP<GPixmap> pm = get_bg_pixmap(rect, subsample, gamma, white);
   // Superpose foreground
-  if (! stencil(pm, rect, subsample, gamma))
+  if (! stencil(pm, rect, subsample, gamma, white))
     // Avoid ugly progressive display (hack)
     if (get_fgjb()) return 0;
   // Return
@@ -1068,10 +1083,18 @@ DjVuImage::get_pixmap(const GRect &rect, int subsample, double gamma) const
 }
 
 
+GP<GPixmap>
+DjVuImage::get_pixmap(const GRect &rect, int subsample, 
+                      double gamma) const
+{
+  return get_pixmap(rect, subsample, gamma, GPixel::WHITE);
+}
+
+
 //// DJVUIMAGE: RENDERING (ARBITRARY SCALE)
 
-typedef GP<GBitmap>(DjVuImage::*BImager)(const GRect &, int, int) const;
-typedef GP<GPixmap>(DjVuImage::*PImager)(const GRect &, int, double) const;
+typedef GP<GBitmap>(DjVuImage::*BImager)(const GRect&,int,int) const;
+typedef GP<GPixmap>(DjVuImage::*PImager)(const GRect&,int,double,GPixel) const;
 
 static GP<GBitmap>
 do_bitmap(const DjVuImage &dimg, BImager get,
@@ -1139,7 +1162,8 @@ do_bitmap(const DjVuImage &dimg, BImager get,
 
 static GP<GPixmap>
 do_pixmap(const DjVuImage &dimg, PImager get,
-          const GRect &inrect, const GRect &inall, double gamma )
+          const GRect &inrect, const GRect &inall, 
+          double gamma, GPixel white )
 {
   GRect rect=inrect;
   GRect all=inall;
@@ -1168,7 +1192,7 @@ do_pixmap(const DjVuImage &dimg, PImager get,
   for (red=1; red<=15; red++)
     if (rw*red>w-red && rw*red<w+red && rh*red>h-red && rh*red<h+red)
     {
-        GP<GPixmap> pm = (dimg.*get)(zrect, red, gamma);
+      GP<GPixmap> pm = (dimg.*get)(zrect, red, gamma, white);
         if( pm ) 
             return pm->rotate(dimg.get_rotate());
         else
@@ -1192,7 +1216,7 @@ do_pixmap(const DjVuImage &dimg, PImager get,
   // Scale
   GRect srect;
   ps.get_input_rect(zrect, srect);
-  GP<GPixmap> spm = (dimg.*get)(srect, red, gamma);
+  GP<GPixmap> spm = (dimg.*get)(srect, red, gamma, white);
   if (!spm) return 0;
   GP<GPixmap> pm = GPixmap::create();
   ps.scale(srect, *spm, zrect, *pm);
@@ -1203,27 +1227,45 @@ do_pixmap(const DjVuImage &dimg, PImager get,
 }
 
 GP<GPixmap>  
-DjVuImage::get_pixmap(const GRect &rect, const GRect &all, double gamma) const
+DjVuImage::get_pixmap(const GRect &r, const GRect &a, double g, GPixel w) const
 {
-  return do_pixmap(*this, & DjVuImage::get_pixmap, rect, all, gamma);
+  return do_pixmap(*this, &DjVuImage::get_pixmap, r, a, g, w);
+}
+
+GP<GPixmap>  
+DjVuImage::get_pixmap(const GRect &r, const GRect &a, double g) const
+{
+  return do_pixmap(*this, &DjVuImage::get_pixmap, r, a, g, GPixel::WHITE);
 }
 
 GP<GBitmap>  
 DjVuImage::get_bitmap(const GRect &rect, const GRect &all, int align) const
 {
-  return do_bitmap(*this, & DjVuImage::get_bitmap, rect, all, align);
+  return do_bitmap(*this, &DjVuImage::get_bitmap, rect, all, align);
 }
 
 GP<GPixmap>  
-DjVuImage::get_bg_pixmap(const GRect &rect, const GRect &all, double gamma) const
+DjVuImage::get_bg_pixmap(const GRect&r, const GRect&a, double g, GPixel w) const
 {
-  return do_pixmap(*this, & DjVuImage::get_bg_pixmap, rect, all, gamma);
+  return do_pixmap(*this, &DjVuImage::get_bg_pixmap, r, a, g, w);
 }
 
 GP<GPixmap>  
-DjVuImage::get_fg_pixmap(const GRect &rect, const GRect &all, double gamma) const
+DjVuImage::get_bg_pixmap(const GRect&r, const GRect&a, double g) const
 {
-  return do_pixmap(*this, & DjVuImage::get_fg_pixmap, rect, all, gamma);
+  return do_pixmap(*this, &DjVuImage::get_bg_pixmap, r, a, g, GPixel::WHITE);
+}
+
+GP<GPixmap>  
+DjVuImage::get_fg_pixmap(const GRect&r, const GRect&a, double g, GPixel w) const
+{
+  return do_pixmap(*this, &DjVuImage::get_fg_pixmap, r, a, g, w);
+}
+
+GP<GPixmap>  
+DjVuImage::get_fg_pixmap(const GRect&r, const GRect&a, double g) const
+{
+  return do_pixmap(*this, &DjVuImage::get_fg_pixmap, r, a, g, GPixel::WHITE);
 }
 
 int 
