@@ -88,16 +88,12 @@
 #endif
 #include <ctype.h>
 
+#ifndef LC_NUMERIC          //MingW
+# undef DO_CHANGELOCALE
+# define LC_NUMERIC 0
+#endif
 #ifndef DO_CHANGELOCALE
-#define DO_CHANGELOCALE 1
-#ifdef UNIX
-#if THREADMODEL != COTHREADS
-#if THREADMODEL != NOTHREADS
-#undef DO_CHANGELOCALE
-#define DO_CHANGELOCALE 0
-#endif
-#endif 
-#endif
+# define DO_CHANGELOCALE 0
 #endif
 
 
@@ -540,25 +536,23 @@ GStringRep::Native::UCS4toString(
 // the results with a nill termination.  (Normally 7 characters
 // is enough.)
 unsigned char *
-GStringRep::UCS4toNative(
-  const uint32_t w0,unsigned char *ptr, mbstate_t *ps)
+GStringRep::UCS4toNative(const uint32_t w0,unsigned char *ptr, mbstate_t *ps)
 {
   uint16_t w1;
   uint16_t w2=1;
-  for(int count=(sizeof(wchar_t)==sizeof(w1)) ? UCS4toUTF16(w0,w1,w2) : 1;
+  for(int count=(sizeof(wchar_t)==sizeof(w1)) 
+        ? UCS4toUTF16(w0,w1,w2) : 1;
       count;
       --count,w1=w2)
-  {
-    // wchar_t can be either UCS4 or UCS2
-    const wchar_t w=(sizeof(wchar_t) == sizeof(w1))?(wchar_t)w1:(wchar_t)w0;
-    int i=wcrtomb((char *)ptr,w,ps);
-    if(i<0)
     {
-      break;
+      // wchar_t can be either UCS4 or UCS2
+      const wchar_t w=(sizeof(wchar_t) == sizeof(w1))?(wchar_t)w1:(wchar_t)w0;
+      int i=wcrtomb((char *)ptr,w,ps);
+      if(i<0)
+        break;
+      ptr[i]=0;
+      ptr += i;
     }
-    ptr[i]=0;
-    ptr += i;
-  }
   ptr[0]=0;
   return ptr;
 }
@@ -1400,7 +1394,8 @@ GStringRep::toEscaped( const bool tosevenbit ) const
   char const *s=start;
   char const *last=s;
   GP<GStringRep> special;
-  for(unsigned long w;(w=getValidUCS4(s));last=s)
+  for(unsigned long w;(w=getValidUCS4(s));last=s)  
+    // Whoever wrote this for statement should be __complete_here__
   {
     char const *ss=0;
     switch(w)
@@ -1800,41 +1795,43 @@ GStringRep::UTF8::UCS4toString(
 }
 
 int
-GStringRep::UTF8::ncopy(
-  wchar_t * const buf, const int buflen ) const
+GStringRep::UTF8::ncopy(wchar_t * const buf, const int buflen ) const
 {
   int retval=(-1);
   if(buf && buflen)
-  {
-	buf[0]=0;
-    if(data[0])
+    {
+      buf[0]=0;
+      if(data[0])
 	{
-      const size_t length=strlen(data);
-      const unsigned char * const eptr=(const unsigned char *)(data+length);
+          const size_t length=strlen(data);
+          const unsigned char * const eptr=(const unsigned char *)(data+length);
 	  wchar_t *r=buf;
 	  wchar_t const * const rend=buf+buflen;
-      for(const unsigned char *s=(const unsigned char *)data;(r<rend)&&(s<eptr)&&*s;)
-	  {
-        const uint32_t w0=UTF8toUCS4(s,eptr);
-        uint16_t w1;
-        uint16_t w2=1;
-        for(int count=(sizeof(wchar_t) == sizeof(w1))?UCS4toUTF16(w0,w1,w2):1;
-            count&&(r<rend);
-            --count,w1=w2,++r)
+          for(const unsigned char *s=(const unsigned char *)data;
+              (r<rend)&&(s<eptr)&&*s;)
+            {
+              const uint32_t w0=UTF8toUCS4(s,eptr);
+              uint16_t w1;
+              uint16_t w2=1;
+              for(int count=(sizeof(wchar_t)==sizeof(w1))
+                    ?UCS4toUTF16(w0,w1,w2):1;
+                  count&&(r<rend);
+                  --count,w1=w2,++r)
 		{
 		  r[0]=(sizeof(wchar_t) == sizeof(w1))?(wchar_t)w1:(wchar_t)w0;
 		}
-	  }
+            }
 	  if(r<rend)
-	  {
-	    r[0]=0;
-		retval=((size_t)r-(size_t)buf)/sizeof(wchar_t);
-	  }
-	}else
+            {
+              r[0]=0;
+              retval=((size_t)r-(size_t)buf)/sizeof(wchar_t);
+            }
+	}
+      else
 	{
 	  retval=0;
 	}
-  }
+    }
   return retval;
 }
 
@@ -1852,26 +1849,36 @@ GStringRep::UTF8::toNative(const EscapeMode escape) const
     mbstate_t ps;
     memset(&ps,0,sizeof(mbstate_t));
     for(const unsigned char *s=(const unsigned char *)data;(s<eptr)&& *s;)
-    {
-      const uint32_t w0=UTF8toUCS4(s,eptr);
-      const unsigned char * const r0=r;
-      r=UCS4toNative(w0,r,&ps);
-      if(r == r0)
       {
-        if(escape == IS_ESCAPED)
-        {
-          sprintf((char *)r,"&#%lu;",(unsigned long)w0);
-          r+=strlen((char *)r);
-        }
+        const unsigned char * const s0 = s;
+        const uint32_t w0=UTF8toUCS4(s,eptr);
+        if (s == s0)
+          {
+            s += 1;
+            *r++ = '?';
+          }
         else
-        {
-          *r++ = '?';
-        }
+          {
+            const unsigned char * const r0 = r;
+            r=UCS4toNative(w0,r,&ps);
+            if(r == r0)
+              {
+                if (escape == IS_ESCAPED)
+                  {
+                    sprintf((char *)r,"&#%lu;",(unsigned long)w0);
+                    r += strlen((char *)r);
+                  }
+                else
+                  {
+                    *r++ = '?';
+                  }
+              }
+          }
       }
-    }
     r[0]=0;
     retval = NATIVE_CREATE( (const char *)buf );
-  } else
+  } 
+  else
   {
     retval = NATIVE_CREATE( (unsigned int)0 );
   }
@@ -2138,19 +2145,6 @@ GStringRep::concat(const GP<GStringRep> &s1,const GP<GStringRep> &s2) const
   return retval;
 }
 
-#ifdef WIN32
-static const char *setlocale_win32(void)
-{
-  static const char *locale=setlocale(LC_ALL,0);
-  if(! locale || (locale[0] == 'C' && !locale[1]))
-  {
-    locale=setlocale(LC_ALL,"");
-  }
-  return locale;
-}
-const char *setlocale_win32_var = setlocale_win32();
-#endif
-
 GStringRep::GStringRep(void)
 {
   size=0;
