@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: cpaldjvu.cpp,v 1.16 2008/05/05 02:24:40 leonb Exp $
-// $Name: release_3_5_22 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -97,8 +94,7 @@
     Simple encoder for low resolution, low color images.
     @author
     L\'eon Bottou <leonb@research.att.com>
-    @version
-    #$Id: cpaldjvu.cpp,v 1.16 2008/05/05 02:24:40 leonb Exp $# */
+*/
 //@{
 //@}
 
@@ -118,12 +114,8 @@
 #include "GOS.h"
 #include "GURL.h"
 #include "DjVuMessage.h"
-
 #include "jb2tune.h"
-
-#include <locale.h>
-#include <stdlib.h>
-#include <math.h>
+#include "common.h"
 
 #ifdef MIN
 #undef MIN
@@ -872,8 +864,7 @@ usage()
 int 
 main(int argc, const char **argv)
 {
-  setlocale(LC_ALL,"");
-  djvu_programname(argv[0]);
+  DJVU_LOCALE;
   GArray<GUTF8String> dargv(0,argc-1);
   for(int i=0;i<argc;++i)
     dargv[i]=GNativeString(argv[i]);
