@@ -158,12 +158,8 @@
 #include "BSByteStream.h"
 
 #include "miniexp.h"
-
 #include "jb2tune.h"
-
-#include <locale.h>
-#include <stddef.h>
-#include <stdlib.h>
+#include "common.h"
 
 #undef MIN
 #undef MAX
@@ -1734,9 +1730,7 @@ parse_slice(const char *q, csepdjvuopts &opts)
 int 
 main(int argc, const char **argv)
 {
-  setlocale(LC_ALL,"");
-  setlocale(LC_NUMERIC,"C");
-  djvu_programname(argv[0]);
+  DJVU_LOCALE;
   GArray<GUTF8String> dargv(0,argc-1);
   for(int i=0;i<argc;++i)
     dargv[i]=GNativeString(argv[i]);
