@@ -52,8 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: ddjvu.cpp,v 1.39 2009/02/13 09:46:27 leonb Exp $
 
 
 /* Program ddjvu has been rewritten to use the ddjvuapi only.
@@ -64,6 +62,7 @@
 # include "config.h"
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -77,20 +76,14 @@
 # include <sys/types.h>
 # include <unistd.h>
 #endif
+#if defined(WIN32) && !defined(__CYGWIN32__)
+# include <io.h>
+# include <mbctype.h>
+#endif
 
 #include "libdjvu/ddjvuapi.h"
 #include "tiff2pdf.h"
 
-#if defined(WIN32) && !defined(__CYGWIN32__)
-# include <io.h>
-# define dup     _dup
-# define lseek   _lseek
-# define open    _open
-# define close   _close
-# define mktemp  _mktemp
-# define O_CREAT _O_CREAT
-# define O_RDWR  _O_RDWR
-#endif
 
 #if HAVE_PUTC_UNLOCKED
 # undef putc
@@ -134,22 +127,25 @@ int          flag_segment = 0;
 int          flag_verbose = 0;
 char         flag_mode = 0;     /* 'c', 'k', 's', 'f','b' */
 char         flag_format = 0;   /* '4','5','6','p','r','t', 'f' */
-int          flag_quality = -1;
+int          flag_quality = -1; /* 1-100 jpg, 900 zip, 901 lzw, 1000 raw */
+int          flag_skipcorrupted = 0;
+int          flag_eachpage = 0;
 const char  *flag_pagespec = 0; 
 ddjvu_rect_t info_size;
 ddjvu_rect_t info_segment;
+const char  *programname = 0;
 const char  *inputfilename = 0;
 const char  *outputfilename = 0;
 
+char *pagefilename = 0;
 #if HAVE_TIFF2PDF
 char *tempfilename = 0;
 int tiffd = -1;
 #endif
-
-FILE *fout = 0;
 #if HAVE_TIFF
 TIFF *tiff = 0;
 #endif
+FILE *fout = 0;
 
 
 
@@ -172,7 +168,6 @@ handle(int wait)
           if (msg->m_error.filename)
             fprintf(stderr,"ddjvu: '%s:%d'\n", 
                     msg->m_error.filename, msg->m_error.lineno);
-          exit(10);
         default:
           break;
         }
@@ -247,7 +242,7 @@ render(ddjvu_page_t *page, int pageno)
   int dpi = ddjvu_page_get_resolution(page);
   ddjvu_page_type_t type = ddjvu_page_get_type(page);
   char *image = 0;
-  char white = 0xFF;
+  char white = (char)0xFF;
   int rowsize;
 #if HAVE_TIFF
   int compression = COMPRESSION_NONE;
@@ -318,7 +313,7 @@ render(ddjvu_page_t *page, int pageno)
   else if (flag_format == 'r' || flag_format == '4')
     mode = DDJVU_RENDER_BLACK;
 
-  /* Determine output pixel format */
+  /* Determine output pixel format and compression */
   style = DDJVU_FORMAT_RGB24;
   if (mode==DDJVU_RENDER_BLACK ||
       mode==DDJVU_RENDER_MASKONLY ||
@@ -330,10 +325,12 @@ render(ddjvu_page_t *page, int pageno)
     }
   switch(flag_format)
     {
-    case 't':
     case 'f':
+    case 't':
 #if HAVE_TIFF
       compression = COMPRESSION_NONE;
+      if (flag_quality >= 1000)
+        break;
 # ifdef CCITT_SUPPORT
       if (style==DDJVU_FORMAT_MSBTOLSB 
           && TIFFFindCODEC(COMPRESSION_CCITT_T6))
@@ -341,19 +338,33 @@ render(ddjvu_page_t *page, int pageno)
 # endif
 # ifdef JPEG_SUPPORT
       if (compression == COMPRESSION_NONE 
-          && style!=DDJVU_FORMAT_MSBTOLSB && flag_quality>0 
+          && style!=DDJVU_FORMAT_MSBTOLSB 
+          && flag_quality>0 && flag_quality<=100
           && TIFFFindCODEC(COMPRESSION_JPEG))
         compression = COMPRESSION_JPEG;
 # endif
-# ifdef PACKBITS_SUPPORT
-      if (compression == COMPRESSION_NONE && flag_format == 't' 
-          && TIFFFindCODEC(COMPRESSION_PACKBITS))
-        compression = COMPRESSION_PACKBITS;
-# endif
 # ifdef ZIP_SUPPORT
-      else if (compression == COMPRESSION_NONE
-               && TIFFFindCODEC(COMPRESSION_DEFLATE))
+      if (compression == COMPRESSION_NONE
+          && (flag_format == 'f' || flag_quality == 900)
+          && TIFFFindCODEC(COMPRESSION_DEFLATE))
+        /* All pdf engines understand deflate. */
         compression = COMPRESSION_DEFLATE;
+# endif
+# ifdef LZW_SUPPORT
+      if (compression == COMPRESSION_NONE
+          && flag_quality == 901
+          && TIFFFindCODEC(COMPRESSION_LZW))
+        /* Because of patents that are now expired, some versions
+           of libtiff only support lzw decoding and trigger an error
+           condition when trying to encode. Unfortunately we cannot
+           know this in advance and select another compression scheme. */
+        compression = COMPRESSION_LZW;
+# endif
+# ifdef PACKBITS_SUPPORT
+      if (compression == COMPRESSION_NONE 
+          && TIFFFindCODEC(COMPRESSION_PACKBITS))
+        /* This mediocre default produces the most portable tiff files. */
+        compression = COMPRESSION_PACKBITS;
 # endif
       break;
 #endif      
@@ -549,6 +560,168 @@ render(ddjvu_page_t *page, int pageno)
 
 
 
+void 
+openfile(int pageno)
+{
+  /* Compute filename */
+  const char *filename = outputfilename;
+  if (flag_eachpage)
+    {
+      sprintf(pagefilename, filename, pageno);
+      filename = pagefilename;
+    }
+  
+  /* Open */
+  if (flag_format == 't') /* tiff file */
+    { 
+#if HAVE_TIFF
+      if (tiff) 
+        {
+          if (! TIFFWriteDirectory(tiff))
+            die(i18n("Problem writing TIFF directory."));
+        }
+      else
+        {
+          if (! strcmp(filename,"-"))
+            die(i18n("TIFF output requires a valid output file name."));
+          if (! (tiff = TIFFOpen(filename, "w")))
+            die(i18n("Cannot open output tiff file '%s'."), filename);
+        }
+#else
+      die(i18n("TIFF output is not compiled"));
+#endif
+    }
+  else if (flag_format == 'f') /* temporary tiff, later converted to pdf */
+    {
+#if HAVE_TIFF2PDF
+      if (tiff) 
+        {
+          if (! TIFFWriteDirectory(tiff))
+            die(i18n("Problem writing directory in temporary TIFF file."));
+        }
+      else
+        {
+          if (! strcmp(filename,"-"))
+            die(i18n("PDF output requires a valid output file name."));
+          if (! (tempfilename = (char*)malloc(strlen(outputfilename) + 8)))
+            die(i18n("Out of memory."));
+          strcpy(tempfilename, outputfilename);
+          strcat(tempfilename, ".XXXXXX");
+          tiff = 0;
+# ifdef WIN32
+          if (_mktemp(tempfilename))
+            tiff = TIFFOpen(tempfilename,"w");
+# elif HAVE_MKSTEMP
+          if ((tiffd = mkstemp(tempfilename)) >= 0)
+            tiff = TIFFFdOpen(tiffd, tempfilename, "w");
+# else
+          if (mktemp(tempfilename))
+            if ((tiffd = open(tempfilename, O_RDWR|O_CREAT)) >= 0)
+              tiff = TIFFFdOpen(tiffd, tempfilename, "w");
+# endif
+          if (! tiff)
+            die(i18n("Cannot create temporary TIFF file '%s'."), tempfilename);
+        }
+#else
+      die(i18n("PDF output is not compiled"));
+#endif
+    } 
+  else if (! fout) /* file output */
+    {
+      if (! strcmp(filename,"-")) {
+        fout = stdout;
+#if defined(__CYGWIN32__)
+        setmode(fileno(fout), O_BINARY);
+#elif defined(WIN32)
+        _setmode(_fileno(fout), _O_BINARY);
+#endif
+      } else if (! (fout = fopen(filename, "wb")))
+        die(i18n("Cannot open output file '%s'."), filename);
+    }
+}
+
+void
+closefile(int pageno)
+{
+  /* Do not close when generating a single file */
+  if (pageno > 0 && ! flag_eachpage)
+    return;
+
+  /* Compute filename */
+  const char *filename = outputfilename;
+  if (flag_eachpage && pageno > 0)
+    {
+      sprintf(pagefilename, filename, pageno);
+      filename = pagefilename;
+    }
+
+  /* Close temporary tiff and generate pdf */
+#if HAVE_TIFF2PDF
+  if (tiff && tempfilename)
+    {
+      const char *args[3];
+      if (! TIFFFlush(tiff))
+        die(i18n("Error while flushing TIFF file."));
+      if (flag_verbose)
+        fprintf(stderr,i18n("Converting temporary TIFF to PDF.\n"));
+#ifndef WIN32
+      if (tiffd >= 0)
+        {
+          int fd = dup(tiffd);
+          TIFFClose(tiff);
+          close(tiffd);
+          tiffd = fd;
+          lseek(tiffd, 0, SEEK_SET);
+          if (! (tiff = TIFFFdOpen(tiffd, tempfilename, "r")))
+            die(i18n("Cannot reopen temporary TIFF file '%s'."), tempfilename);
+        }
+      else
+#endif
+        {
+          TIFFClose(tiff);
+          if (!(tiff = TIFFOpen(tempfilename, "r")))
+            die(i18n("Cannot reopen temporary TIFF file '%s'."), tempfilename);
+        }
+      // Convert
+      if (! (fout = fopen(filename, "wb")))
+        die(i18n("Cannot open output file '%s'."), filename);
+      args[0] = programname;
+      args[1] = "-o";
+      args[2] = filename;
+      if (tiff2pdf(tiff, fout, 3, args) != EXIT_SUCCESS)
+        die(i18n("Error occured while creating PDF file."));
+      TIFFClose(tiff);
+      tiff = 0;
+#ifndef WIN32
+      close(tiffd);
+      tiffd = -1;
+#endif
+      remove(tempfilename);
+      free(tempfilename);
+      tempfilename = 0;
+    }
+#endif
+  /* Close tiff */
+#if HAVE_TIFF
+  if (tiff)
+    {
+      if (! TIFFFlush(tiff))
+        die(i18n("Error while flushing tiff file."));
+      TIFFClose(tiff);
+      tiff = 0;
+    }
+#endif
+  /* Close fout */
+  if (fout)
+    {
+      if (fflush(fout) < 0)
+        die(i18n("Error while flushing output file: %s"), strerror(errno));
+      fclose(fout);
+      fout = 0;
+    }
+}
+
+
 void
 dopage(int pageno)
 {
@@ -560,74 +733,30 @@ dopage(int pageno)
   while (! ddjvu_page_decoding_done(page))
     handle(TRUE);
   if (ddjvu_page_decoding_error(page))
-    die(i18n("Cannot decode page %d."), pageno);
-  
+    {
+      handle(FALSE);
+      fprintf(stderr,"ddjvu: ");
+      fprintf(stderr,i18n("Cannot decode page %d."), pageno);
+      fprintf(stderr,"\n");
+      if (flag_skipcorrupted)
+        return;
+      else
+        exit(10);
+    }
   timingdata[1] = ticks();
-  /* Open files */
-  if (flag_format == 't')
+  /* Create filename */
+  if (flag_eachpage)
     {
-#if HAVE_TIFF
-      if (tiff) 
-        {
-          if (! TIFFWriteDirectory(tiff))
-            die(i18n("Problem writing TIFF directory."));
-        }
-      else
-        {
-          if (! strcmp(outputfilename,"-"))
-            die(i18n("TIFF output requires a valid output file name."));
-          if (! (tiff = TIFFOpen(outputfilename, "w")))
-            die(i18n("Cannot open output tiff file '%s'."), outputfilename);
-        }
-#else
-      die(i18n("TIFF output is not compiled"));
-#endif
-    }
-  else if (flag_format == 'f')  
-    {
-#if HAVE_TIFF2PDF
-      if (tiff) 
-        {
-          if (! TIFFWriteDirectory(tiff))
-            die(i18n("Problem writing TIFF directory."));
-        }
-      else
-        {
-          if (! strcmp(outputfilename,"-"))
-            die(i18n("PDF output requires a valid output file name."));
-          if (! (tempfilename = (char*)malloc(strlen(outputfilename) + 8)))
-            die(i18n("Out of memory."));
-          strcpy(tempfilename, outputfilename);
-          strcat(tempfilename, ".XXXXXX");
-# if HAVE_MKSTEMP
-          tiffd = mkstemp(tempfilename);
-# else
-          if (mktemp(tempfilename))
-            tiffd = open(tempfilename, O_RDWR|O_CREAT);
-# endif
-          if (tiffd < 0 || ! (tiff = TIFFFdOpen(tiffd, tempfilename, "w")))
-            die(i18n("Cannot create temporary tiff file '%s'."), tempfilename);
-        }
-#else
-      die(i18n("PDF output is not compiled"));
-#endif
-    } 
-  else if (! fout) 
-    {
-      if (! strcmp(outputfilename,"-")) {
-        fout = stdout;
-#if defined(__CYGWIN32__)
-        setmode(fileno(fout), O_BINARY);
-#elif defined(WIN32)
-        _setmode(_fileno(fout), _O_BINARY);
-#endif
-      } else if (! (fout = fopen(outputfilename, "wb")))
-        die(i18n("Cannot open output file '%s'."), outputfilename);
+      
     }
+
+
   /* Render */
+  openfile(pageno);
   inform(page, pageno);
   render(page, pageno);
   ddjvu_page_release(page);
+  closefile(pageno);
 }
 
 
@@ -681,10 +810,10 @@ parse_pagespec(const char *s, int max_page, void (*dopage)(int))
         p += 1;
       if (! spec)
         die(i18n(err), s);
-      if (end_page < 0)
-        end_page = 0;
-      if (start_page < 0)
-        start_page = 0;
+      if (end_page <= 0)
+        end_page = 1;
+      if (start_page <= 0)
+        start_page = 1;
       if (end_page > max_page)
         end_page = max_page;
       if (start_page > max_page)
@@ -753,19 +882,21 @@ usage()
     i18n("DjVu decompression utility\n\n"
          "Usage: ddjvu [options] [<djvufile> [<outputfile>]]\n\n"
          "Options:\n"
-         "  -verbose          Prints various informational messages.\n"
-         "  -format=FMT       Selects output format: pbm,pgm,ppm,pnm,rle,tiff.\n"
-         "  -scale=N          Selects display scale.\n"
-         "  -size=WxH         Selects size of rendered image.\n"
-         "  -subsample=N      Selects direct subsampling factor.\n"
-         "  -aspect=no        Authorizes aspect ratio changes\n"
-         "  -segment=WxH+X+Y  Selects which segment of the rendered image\n"
-         "  -mode=black       Renders a meaningful bitonal image.\n"
-         "  -mode=mask        Only renders the mask layer.\n"
-         "  -mode=foreground  Only renders the foreground layer.\n"
-         "  -mode=background  Only renders the background layer.\n"
-         "  -page=PAGESPEC    Selects page(s) to be decoded.\n"
-         "  -quality=QUALITY  Specifies jpeg quality for lossy tiff output.\n"
+         "  -verbose          Print various informational messages.\n"
+         "  -format=FMT       Select output format: pbm,pgm,ppm,pnm,rle,tiff.\n"
+         "  -scale=N          Select display scale.\n"
+         "  -size=WxH         Select size of rendered image.\n"
+         "  -subsample=N      Select direct subsampling factor.\n"
+         "  -aspect=no        Authorize aspect ratio changes\n"
+         "  -segment=WxH+X+Y  Select which segment of the rendered image\n"
+         "  -mode=black       Render a meaningful bitonal image.\n"
+         "  -mode=mask        Only render the mask layer.\n"
+         "  -mode=foreground  Only render the foreground layer.\n"
+         "  -mode=background  Only render the background layer.\n"
+         "  -page=PAGESPEC    Select page(s) to be decoded.\n"
+         "  -skip             Skip corrupted pages instead of aborting.\n"
+         "  -eachpage         Produce one file per page (using %d in outputfile).\n"
+         "  -quality=QUALITY  Specify jpeg quality for lossy tiff output.\n"
          "\n"
          "If <outputfile> is a single dash or omitted, the decompressed image\n"
          "is sent to the standard output.  If <djvufile> is a single dash or\n"
@@ -837,7 +968,6 @@ parse_option(int argc, char **argv, int i)
       arg = opt;
       opt = "subsample";
     }
-  
   /* Parse options */
   if (!strcmp(opt,"v") ||
       !strcmp(opt,"verbose"))
@@ -846,6 +976,18 @@ parse_option(int argc, char **argv, int i)
         die(i18n(errarg), opt);
       flag_verbose = 1;
     }
+  else if (!strcmp(opt,"skip"))
+    {
+      if (arg) 
+        die(i18n(errarg), opt);
+      flag_skipcorrupted = 1;
+    }
+  else if (! strcmp(opt,"eachpage"))
+    {
+      if (arg) 
+        die(i18n(errarg), opt);
+      flag_eachpage = 1;
+    }
   else if (!strcmp(opt,"scale"))
     {
       if (!arg) 
@@ -957,7 +1099,13 @@ parse_option(int argc, char **argv, int i)
         die(i18n(errdupl), opt);
       else if (!arg) 
         flag_quality = 100;
-      else 
+      else if (!strcmp(arg,"deflate") || !strcmp(arg,"zip"))
+        flag_quality = 900;
+      else if (!strcmp(arg,"lzw"))
+        flag_quality = 901;
+      else if (!strcmp(arg,"uncompressed") || !strcmp(arg,"raw"))
+        flag_quality = 1000;
+      else
         {
           flag_quality = strtol(arg,&end,10);
           if (*end || flag_quality<25 || flag_quality>150)
@@ -976,12 +1124,48 @@ parse_option(int argc, char **argv, int i)
 }
 
 
+int
+check_eachpage(const char *s)
+{
+  const char *p = s;
+  int hasd = 0;
+  int size = 0;
+  char c;
+  while ((c = *s++))
+    if (c == '%')
+      {
+        c = *s++;
+        if (c == '%')
+          continue;
+        if (hasd)
+          return 0;
+        if (c == '-' || c == '+' || c == ' ')
+          c = *s++;
+        while (c >= '0' && c <= '9')
+          {
+            size = 10 * size + c - '0';
+            c = *s++;
+          }
+        if (c != 'd')
+          return 0;
+        hasd = 1;
+      }
+  if (size == 0)
+    size = 30;
+  if (hasd == 1 && size>0 && size<1000)
+    return size + s - p;
+  return 0;
+}
+
 
 int
 main(int argc, char **argv)
 {
-  /* Parse options */
   int i;
+#if defined(WIN32) && !defined(__CYGWIN32__)
+  _setmbcp(_MB_CP_OEM);
+#endif
+  /* Parse options */
   for (i=1; i<argc; i++)
     {
       char *s = argv[i];
@@ -994,7 +1178,7 @@ main(int argc, char **argv)
       else
         usage();
     }
-  
+
   /* Defaults */
   if (! inputfilename)
     inputfilename = "-";
@@ -1002,67 +1186,35 @@ main(int argc, char **argv)
     outputfilename = "-";
   if (! flag_pagespec)
     flag_pagespec = (flag_format) ? "1-$" : "1";
-
+  if (flag_eachpage)
+    {
+      int sz = check_eachpage(outputfilename);
+      if (! sz)
+        die(i18n("Flag -eachpage demands a '%%d' specification in the output file name."));
+      pagefilename = (char*)malloc(sz);
+      if (! pagefilename)
+        die(i18n("Out of memory"));
+    }
+  
   /* Create context and document */
-  if (! (ctx = ddjvu_context_create(argv[0])))
+  programname = argv[0];
+  if (! (ctx = ddjvu_context_create(programname)))
     die(i18n("Cannot create djvu context."));
   if (! (doc = ddjvu_document_create_by_filename(ctx, inputfilename, TRUE)))
     die(i18n("Cannot open djvu document '%s'."), inputfilename);
   while (! ddjvu_document_decoding_done(doc))
     handle(TRUE);
+  if (ddjvu_document_decoding_error(doc))
+    die(i18n("Cannot decode document."));
   
   /* Process all pages */
   i = ddjvu_document_get_pagenum(doc);
   parse_pagespec(flag_pagespec, i, dopage);
 
-  /* Close */
-#if HAVE_TIFF2PDF
-  if (tiff && tiffd >= 0 && tempfilename)
-    {
-      int fd = dup(tiffd);
-      if (! TIFFFlush(tiff))
-        die(i18n("Error while flushing TIFF file."));
-      TIFFClose(tiff);
-      close(tiffd);
-      tiffd = fd;
-      if (flag_verbose)
-        fprintf(stderr,i18n("Converting temporary TIFF to PDF.\n"));
-      lseek(tiffd, 0, SEEK_SET);
-      if (! (tiff = TIFFFdOpen(tiffd, tempfilename, "r")))
-        die(i18n("Cannot reopen temporary tiff file '%s'."), tempfilename);
-      if (! (fout = fopen(outputfilename, "wb")))
-        die(i18n("Cannot open output file '%s'."), outputfilename);
-      const char *args[3];
-      args[0] = argv[0];
-      args[1] = "-o";
-      args[2] = outputfilename;
-      if (tiff2pdf(tiff, fout, 3, args) != EXIT_SUCCESS)
-        die(i18n("Error occured while creating PDF file."));
-      TIFFClose(tiff);
-      tiff = 0;
-      close(tiffd);
-      tiffd = -1;
-      remove(tempfilename);
-      free(tempfilename);
-      tempfilename = 0;
-    }
-#endif
-#if HAVE_TIFF
-  if (tiff)
-    {
-      if (! TIFFFlush(tiff))
-        die(i18n("Error while flushing tiff file."));
-      TIFFClose(tiff);
-      tiff = 0;
-    }
-#endif
-  if (fout)
-    {
-      if (fflush(fout) < 0)
-        die(i18n("Error while flushing output file: %s"), strerror(errno));
-      fclose(fout);
-      fout = 0;
-    }
+  /* Close output file */
+  closefile(0);
+
+  /* Release */
   if (doc)
     ddjvu_document_release(doc);
   if (ctx)
