@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: djvmcvt.cpp,v 1.10 2010/05/27 20:47:57 leonb Exp $
-// $Name: release_3_5_23 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -135,8 +132,7 @@
     DjVu multipage document converter.
     @author
     Andrei Erofeev <eaf@geocities.com>
-    @version
-    #$Id: djvmcvt.cpp,v 1.10 2010/05/27 20:47:57 leonb Exp $# */
+*/
 
 
 #include "DjVuDocument.h"
@@ -145,10 +141,7 @@
 #include "GOS.h"
 #include "DjVuMessage.h"
 #include "debug.h"
-
-#include <locale.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include "common.h"
 
 static const char * progname;
 
@@ -206,9 +199,7 @@ do_indirect(GArray<GUTF8String> &argv)
 int 
 main(int argc, char ** argv)
 {
-  setlocale(LC_ALL,"");
-  setlocale(LC_NUMERIC,"C");
-  djvu_programname(argv[0]);
+  DJVU_LOCALE;
   GArray<GUTF8String> dargv(0,argc-1);
   for(int i=0;i<argc;++i)
     dargv[i]=GNativeString(argv[i]);
