@@ -52,13 +52,12 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: djvups.cpp,v 1.22 2008/07/02 23:16:14 leonb Exp $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -70,6 +69,7 @@
 
 #if defined(WIN32) || defined(__CYGWIN32__)
 # include <io.h>
+# include <mbctype.h>
 #endif
 
 /* Some day we'll redo i18n right. */
@@ -219,13 +219,16 @@ check_option(char *s)
 int
 main(int argc, char **argv)
 {
+
   int i;
   int optc = 0;
   char **optv;
   const char *infile = 0;
   const char *outfile = 0;
   FILE *fout;
-
+#if defined(WIN32) && !defined(__CYGWIN32__)
+  _setmbcp(_MB_CP_OEM);
+#endif
   /* Sort options */
   if (! (optv = (char**)malloc(argc*sizeof(char*))))
     die(i18n("Out of memory"));
